package be.iminds.ilabt.jfed.experiment.setup.config;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;
import java.util.Objects;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class ESpec {
    @Nonnull
    private final ESpecSource source;
    @Nullable
    private final String providedContentSource; //url, file or content as specified in source

    @Nullable private final List<ESpecLogStorageDetails> logStorage;

    public ESpec() {
        this.source = ESpecSource.PROVIDE_DIR;
        this.providedContentSource = null;
        this.logStorage = null;
    }

    @JsonCreator
    public ESpec(@Nullable @JsonProperty("source") ESpecSource source,
                 @Nullable @JsonProperty("providedContentSource") String providedContentSource,
                 @Nullable @JsonProperty("logStorage") List<ESpecLogStorageDetails> logStorage) {
        this.source = source == null ? new ESpec().getSource() : source;
        this.providedContentSource = providedContentSource;
        this.logStorage = logStorage;
    }

    @JsonIgnore
    public boolean isValid() {
        switch (source) {
            case PROVIDE_ARCHIVE_FILE:
            case PROVIDE_ARCHIVE_URL:
            case PROVIDE_GIT_REPO_DIR:
            case DIRECT:
            case PROVIDE_DIR: {
                if (providedContentSource == null)
                    throw new IllegalArgumentException("Config Error: rspec.providedContentSource may not be null if espec.source == "+source);
                return true;
            }
            default:
                throw new IllegalArgumentException("Config Error: unsupported espec.source == "+source);
        }
    }

    @JsonProperty
    @Nonnull
    public ESpecSource getSource() {
        return source;
    }

    @JsonProperty
    @Nullable
    public String getProvidedContentSource() {
        return providedContentSource;
    }

    @JsonProperty
    @Nullable
    public List<ESpecLogStorageDetails> getLogStorage() {
        return logStorage;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof ESpec)) return false;
        ESpec eSpec = (ESpec) o;
        return source == eSpec.source &&
                Objects.equals(providedContentSource, eSpec.providedContentSource) &&
                Objects.equals(logStorage, eSpec.logStorage);
    }

    @Override
    public int hashCode() {
        return Objects.hash(source, providedContentSource, logStorage);
    }

    public enum ESpecSource {PROVIDE_ARCHIVE_URL, PROVIDE_ARCHIVE_FILE, PROVIDE_DIR, PROVIDE_GIT_REPO_DIR, DIRECT }
}
